@@ -1,15 +1,13 @@
-/***********************************************************
-* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
-*----------------------------------------------------------*
-* (c) 1995-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* conf_snd.c                                               *
-***********************************************************/
+// ============================================================================
+// Rocks'n'Diamonds - McDuffin Strikes Back!
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// conf_snd.c
+// ============================================================================
 
 #include "libgame/libgame.h"
 #include "main.h"
@@ -121,6 +119,9 @@ struct ConfigInfo sound_config[] =
   { "magic_wall.activating",		"quirk.wav"			},
   { "magic_wall.active",		"miep.wav"			},
   { "magic_wall.filling",		"quirk.wav"			},
+  { "dc_magic_wall.activating",		"quirk.wav"			},
+  { "dc_magic_wall.active",		"miep.wav"			},
+  { "dc_magic_wall.filling",		"quirk.wav"			},
   { "[amoeba].waiting",			UNDEFINED_FILENAME		},
   { "[amoeba].growing",			"amoebe.wav"			},
   { "[amoeba].dropping",		UNDEFINED_FILENAME		},
@@ -130,6 +131,15 @@ struct ConfigInfo sound_config[] =
   { "[exit].opening",			"oeffnen.wav"			},
   { "[exit].closing",			"oeffnen.wav"			},
   { "[exit].passing",			"buing.wav"			},
+  { "[steel_exit].opening",		"oeffnen.wav"			},
+  { "[steel_exit].closing",		"oeffnen.wav"			},
+  { "[steel_exit].passing",		"buing.wav"			},
+  { "[em_exit].opening",		"gong.wav"			},
+  { "[em_exit].closing",		UNDEFINED_FILENAME		},
+  { "[em_exit].passing",		"buing.wav"			},
+  { "[em_steel_exit].opening",		"gong.wav"			},
+  { "[em_steel_exit].closing",		UNDEFINED_FILENAME		},
+  { "[em_steel_exit].passing",		"buing.wav"			},
   { "penguin.passing",			"buing.wav"			},
 
   /* sounds for Emerald Mine Club style elements and actions */
@@ -170,10 +180,10 @@ struct ConfigInfo sound_config[] =
   { "[switchgate].opening",		"oeffnen.wav"			},
   { "[switchgate].closing",		"oeffnen.wav"			},
   { "[switchgate].passing",		"gate.wav"			},
-  { "timegate_switch.activating",	"deng.wav"			},
-  { "timegate_switch.active",		"miep.wav"			},
-  { "timegate_switch.deactivating",	UNDEFINED_FILENAME		},
-  { "timegate.opening",			"oeffnen.wav"			},
+  { "[timegate_switch].activating",	"deng.wav"			},
+  { "[timegate_switch].active",		"miep.wav"			},
+  { "[timegate_switch].deactivating",	UNDEFINED_FILENAME		},
+  { "[timegate].opening",		"oeffnen.wav"			},
   { "[timegate].closing",		"oeffnen.wav"			},
   { "[timegate].passing",		"gate.wav"			},
   { "[conveyor_belt_switch].activating",UNDEFINED_FILENAME		},
@@ -237,45 +247,50 @@ struct ConfigInfo sound_config[] =
   { "game.sokoban_solving",		"buing.wav"			},
 
   /* sounds for other non-game actions */
-  { "door.opening",			"oeffnen.wav"			},
-  { "door.closing",			"oeffnen.wav"			},
+  { "door.opening",			"door.wav"			},
+  { "door.closing",			"door.wav"			},
+
+  { "request.opening",			UNDEFINED_FILENAME		},
+  { "request.closing",			UNDEFINED_FILENAME		},
 
   /* sounds for menu actions */
   { "menu.item.activating",		"empty.wav"			},
   { "menu.item.selecting",		"base.wav"			},
 
+  { "menu.button.pressing",		UNDEFINED_FILENAME		},
+  { "menu.button.releasing",		UNDEFINED_FILENAME		},
+
+  { "background.TITLE_INITIAL",		UNDEFINED_FILENAME		},
   { "background.TITLE",			UNDEFINED_FILENAME		},
-  { "background.MESSAGE",		UNDEFINED_FILENAME		},
   { "background.MAIN",			UNDEFINED_FILENAME		},
   { "background.LEVELS",		UNDEFINED_FILENAME		},
+  { "background.LEVELNR",		UNDEFINED_FILENAME		},
   { "background.SCORES",		"halloffame.wav"		},
   { "background.SCORES.mode_loop",	"false"				},
   { "background.EDITOR",		UNDEFINED_FILENAME		},
   { "background.INFO",			UNDEFINED_FILENAME		},
   { "background.SETUP",			UNDEFINED_FILENAME		},
 
-#if 0
-  { "[not used]",			"antigrav.wav"			},
-  { "[not used]",			"bong.wav"			},
-  { "[not used]",		 	"fuel.wav"			},
-  { "[not used]",			"holz.wav"			},
-  { "[not used]",			"hui.wav"			},
-  { "[not used]",			"kabumm.wav"			},
-  { "[not used]",			"kink.wav"			},
-  { "[not used]",			"kling.wav"			},
-  { "[not used]",			"krach.wav"			},
-  { "[not used]",			"laser.wav"			},
-  { "[not used]",			"quiek.wav"			},
-  { "[not used]",			"rumms.wav"			},
-  { "[not used]",			"schlopp.wav"			},
-  { "[not used]",			"schrff.wav"			},
-  { "[not used]",			"schwirr.wav"			},
-  { "[not used]",			"slurp.wav"			},
-  { "[not used]",			"sproing.wav"			},
-  { "[not used]",			"warnton.wav"			},
-  { "[not used]",			"whoosh.wav"			},
-  { "[not used]",			"boom.wav"			},
-#endif
+  { "background.titlescreen_initial_1",	UNDEFINED_FILENAME		},
+  { "background.titlescreen_initial_2",	UNDEFINED_FILENAME		},
+  { "background.titlescreen_initial_3",	UNDEFINED_FILENAME		},
+  { "background.titlescreen_initial_4",	UNDEFINED_FILENAME		},
+  { "background.titlescreen_initial_5",	UNDEFINED_FILENAME		},
+  { "background.titlescreen_1",		UNDEFINED_FILENAME		},
+  { "background.titlescreen_2",		UNDEFINED_FILENAME		},
+  { "background.titlescreen_3",		UNDEFINED_FILENAME		},
+  { "background.titlescreen_4",		UNDEFINED_FILENAME		},
+  { "background.titlescreen_5",		UNDEFINED_FILENAME		},
+  { "background.titlemessage_initial_1",UNDEFINED_FILENAME		},
+  { "background.titlemessage_initial_2",UNDEFINED_FILENAME		},
+  { "background.titlemessage_initial_3",UNDEFINED_FILENAME		},
+  { "background.titlemessage_initial_4",UNDEFINED_FILENAME		},
+  { "background.titlemessage_initial_5",UNDEFINED_FILENAME		},
+  { "background.titlemessage_1",	UNDEFINED_FILENAME		},
+  { "background.titlemessage_2",	UNDEFINED_FILENAME		},
+  { "background.titlemessage_3",	UNDEFINED_FILENAME		},
+  { "background.titlemessage_4",	UNDEFINED_FILENAME		},
+  { "background.titlemessage_5",	UNDEFINED_FILENAME		},
 
   { NULL,				NULL				}
 };
