@@ -1,15 +1,13 @@
-/***********************************************************
-* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
-*----------------------------------------------------------*
-* (c) 1995-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* game.h                                                   *
-***********************************************************/
+// ============================================================================
+// Rocks'n'Diamonds - McDuffin Strikes Back!
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// game.h
+// ============================================================================
 
 #ifndef GAME_H
 #define GAME_H
@@ -17,20 +15,32 @@
 /* (not included here due to collisions with Emerald Mine engine definitions) */
 /* #include "main.h" */
 
-#define MAX_INVENTORY_SIZE	1000
+#define MAX_INVENTORY_SIZE		1000
 
-#define STD_NUM_KEYS		4
-#define MAX_NUM_KEYS		8
+#define STD_NUM_KEYS			4
+#define MAX_NUM_KEYS			8
 
-#define NUM_BELTS		4
-#define NUM_BELT_PARTS		3
+#define NUM_BELTS			4
+#define NUM_BELT_PARTS			3
+
+#define NUM_PANEL_INVENTORY		8
+#define NUM_PANEL_GRAPHICS		8
+#define NUM_PANEL_ELEMENTS		8
+#define NUM_PANEL_CE_SCORE		8
+
+#define STR_SNAPSHOT_MODE_OFF		"off"
+#define STR_SNAPSHOT_MODE_EVERY_STEP	"every_step"
+#define STR_SNAPSHOT_MODE_EVERY_MOVE	"every_move"
+#define STR_SNAPSHOT_MODE_EVERY_COLLECT	"every_collect"
+#define STR_SNAPSHOT_MODE_DEFAULT	STR_SNAPSHOT_MODE_OFF
+
+#define SNAPSHOT_MODE_OFF		0
+#define SNAPSHOT_MODE_EVERY_STEP	1
+#define SNAPSHOT_MODE_EVERY_MOVE	2
+#define SNAPSHOT_MODE_EVERY_COLLECT	3
+#define SNAPSHOT_MODE_DEFAULT		SNAPSHOT_MODE_OFF
 
-#define NUM_PANEL_INVENTORY	8
-#define NUM_PANEL_GRAPHICS	8
-#define NUM_PANEL_ELEMENTS	8
-#define NUM_PANEL_CE_SCORE	8
 
-#if 1
 struct GamePanelInfo
 {
   struct TextPosInfo level_number;
@@ -47,6 +57,7 @@ struct GamePanelInfo
   struct TextPosInfo time_hh;
   struct TextPosInfo time_mm;
   struct TextPosInfo time_ss;
+  struct TextPosInfo frame;
   struct TextPosInfo shield_normal;
   struct TextPosInfo shield_normal_time;
   struct TextPosInfo shield_deadly;
@@ -88,26 +99,32 @@ struct GamePanelInfo
 
 struct GameButtonInfo
 {
-  struct MenuPosInfo stop;
-  struct MenuPosInfo pause;
-  struct MenuPosInfo play;
-  struct MenuPosInfo sound_music;
-  struct MenuPosInfo sound_loops;
-  struct MenuPosInfo sound_simple;
+  struct XY stop;
+  struct XY pause;
+  struct XY play;
+
+  struct XY undo;
+  struct XY redo;
+
+  struct XY save;
+  struct XY pause2;
+  struct XY load;
+
+  struct XY sound_music;
+  struct XY sound_loops;
+  struct XY sound_simple;
 };
 
-#else
-
-struct GamePanelInfo
+struct GameSnapshotInfo
 {
-  struct XY level;
-  struct XY gems;
-  struct XY inventory;
-  struct XY keys;
-  struct XY score;
-  struct XY time;
+  int mode;
+
+  byte last_action[MAX_PLAYERS];
+  boolean changed_action;
+  boolean collected_item;
+
+  boolean save_snapshot;
 };
-#endif
 
 struct GameInfo
 {
@@ -116,9 +133,13 @@ struct GameInfo
   struct GameButtonInfo button;
 
   /* values for graphics engine customization */
+  int graphics_engine_version;
   boolean use_native_emc_graphics_engine;
+  boolean use_native_sp_graphics_engine;
+  boolean use_masked_pushing;
   int forced_scroll_delay_value;
   int scroll_delay_value;
+  int tile_size;
 
   /* values for engine initialization */
   int default_push_delay_fixed;
@@ -131,6 +152,10 @@ struct GameInfo
   int initial_move_delay_value[MAX_PLAYERS];
   int initial_push_delay_value;
 
+  /* flag for single or multi-player mode (needed for playing tapes) */
+  /* (when playing/recording games, this is identical to "setup.team_mode" */
+  boolean team_mode;
+
   /* flags to handle bugs in and changes between different engine versions */
   /* (for the latest engine version, these flags should always be "FALSE") */
   boolean use_change_when_pushing_bug;
@@ -152,6 +177,7 @@ struct GameInfo
   boolean gravity;
   boolean explosions_delayed;
   boolean envelope_active;
+  boolean no_time_limit;	/* (variable only in very special case) */
 
   /* values for the new EMC elements */
   int lenses_time_left;
@@ -174,7 +200,10 @@ struct GameInfo
   boolean set_centered_player;
 
   /* values for random number generator initialization after snapshot */
-  unsigned long num_random_calls;
+  unsigned int num_random_calls;
+
+  /* values for game engine snapshot control */
+  struct GameSnapshotInfo snapshot;
 };
 
 struct PlayerInfo
@@ -245,8 +274,15 @@ struct PlayerInfo
   boolean is_bored;
   boolean is_sleeping;
 
+  boolean was_waiting;
+  boolean was_moving;
+  boolean was_snapping;
+  boolean was_dropping;
+
   boolean cannot_move;
 
+  boolean force_dropping;	/* needed for single step mode */
+
   int frame_counter_bored;
   int frame_counter_sleeping;
 
@@ -274,7 +310,7 @@ struct PlayerInfo
   int push_delay;
   int push_delay_value;
 
-  unsigned long actual_frame_counter;
+  unsigned int actual_frame_counter;
 
   int drop_delay;
   int drop_pressed_delay;
@@ -310,6 +346,8 @@ void DEBUG_SetMaximumDynamite();
 void GetPlayerConfig(void);
 int GetElementFromGroupElement(int);
 
+int getPlayerInventorySize(int);
+
 void DrawGameValue_Time(int);
 void DrawGameDoorValues(void);
 
@@ -318,7 +356,7 @@ void UpdateAndDisplayGameControlValues();
 void InitGameSound();
 void InitGame();
 
-void UpdateEngineValues(int, int);
+void UpdateEngineValues(int, int, int, int);
 void GameWon(void);
 void GameEnd(void);
 
@@ -327,16 +365,19 @@ void Moving2Blocked(int, int, int *, int *);
 void Blocked2Moving(int, int, int *, int *);
 void DrawDynamite(int, int);
 
-void StartGameActions(boolean, boolean, long);
+void StartGameActions(boolean, boolean, int);
 
 void GameActions(void);
 void GameActions_EM_Main();
+void GameActions_SP_Main();
+void GameActions_RND_Main();
 void GameActions_RND();
 
 void ScrollLevel(int, int);
 
 void InitPlayLevelSound();
 void PlayLevelSound_EM(int, int, int, int);
+void PlayLevelSound_SP(int, int, int, int);
 
 void RaiseScore(int);
 void RaiseScoreElement(int);
@@ -344,17 +385,27 @@ void RaiseScoreElement(int);
 void RequestQuitGameExt(boolean, boolean, char *);
 void RequestQuitGame(boolean);
 
-unsigned int InitEngineRandom_RND(long);
+unsigned int InitEngineRandom_RND(int);
 unsigned int RND(int);
 
-void FreeEngineSnapshot();
-void LoadEngineSnapshot();
-void SaveEngineSnapshot();
-boolean CheckEngineSnapshot();
+void FreeEngineSnapshotSingle();
+void FreeEngineSnapshotList();
+void LoadEngineSnapshotSingle();
+void SaveEngineSnapshotSingle();
+boolean CheckSaveEngineSnapshotToList();
+void SaveEngineSnapshotToList();
+void SaveEngineSnapshotToListInitial();
+boolean CheckEngineSnapshotSingle();
+boolean CheckEngineSnapshotList();
 
 void CreateGameButtons();
 void FreeGameButtons();
+void MapUndoRedoButtons();
+void UnmapUndoRedoButtons();
+void MapGameButtons();
 void UnmapGameButtons();
 void RedrawGameButtons();
 
+void HandleSoundButtonKeys(Key);
+
 #endif
