@@ -14,10 +14,6 @@ extern int debug;
 extern char *progname;
 extern char *arg_basedir;
 
-#if 0
-extern int TILEX, TILEY;
-#endif
-
 extern int frame;
 
 extern short ulaw_to_linear[256];
@@ -28,18 +24,10 @@ extern unsigned char linear_to_ulaw[65536];
 int open_all(void);
 void close_all(void);
 
-#if 1
 void readjoy(byte, struct PLAYER *);
-#else
-void readjoy(byte);
-#endif
 void input_eventloop(void);
 
-void blitscreen(void);
 void game_initscreen(void);
-void game_animscreen(void);
-
-void DrawGameDoorValues_EM();
 
 void play_sound(int, int, int);
 void sound_play(void);
