@@ -63,46 +63,18 @@ void InitGameEngine_EM()
   prepare_em_level();
 
   game_initscreen();
-  game_animscreen();
 
-#if 0
-  /* blit playfield from scroll buffer to normal back buffer for fading in */
-  BlitScreenToBitmap_EM(backbuffer);
-#endif
+  RedrawPlayfield_EM(FALSE);
+}
+
+void UpdateGameDoorValues_EM()
+{
 }
 
 void GameActions_EM(byte action[MAX_PLAYERS], boolean warp_mode)
 {
   int i;
-  boolean player_is_dropping = FALSE;
-
-#if 0
-  static int foo = -1;
-
-  if (action[0] == 0 && foo != 0)
-    printf("KEY RELEASED @ %05d\n", FrameCounter);
-
-  foo = action[0];
-#endif
-
-#if 0
-#if 1
-  if (FrameCounter % 10 == 0)
-#endif
-    printf("::: %05d: %lu, %d\n", FrameCounter, RandomEM, frame);
-#endif
-
-#if 0
-  game_animscreen();
-
-#if 1
-#if 0
-  SyncDisplay();
-#endif
-
-  blitscreen();
-#endif
-#endif
+  boolean any_player_dropping = FALSE;
 
   RandomEM = RandomEM * 129 + 1;
 
@@ -111,7 +83,7 @@ void GameActions_EM(byte action[MAX_PLAYERS], boolean warp_mode)
   for (i = 0; i < MAX_PLAYERS; i++)
     readjoy(action[i], &ply[i]);
 
-  UpdateEngineValues(screen_x / TILEX, screen_y / TILEY);
+  UpdateEngineValues(screen_x / TILEX, screen_y / TILEY, ply[0].x, ply[0].y);
 
   if (frame == 7)
   {
@@ -124,8 +96,7 @@ void GameActions_EM(byte action[MAX_PLAYERS], boolean warp_mode)
     synchro_3();
     sound_play();
 
-    if (!warp_mode)		/* do not redraw values in warp mode */
-      DrawGameDoorValues_EM();
+    UpdateGameDoorValues_EM();
   }
 
   for (i = 0; i < MAX_PLAYERS; i++)
@@ -133,22 +104,12 @@ void GameActions_EM(byte action[MAX_PLAYERS], boolean warp_mode)
 	ply[i].dynamite &&
 	ply[i].dynamite_cnt > 0 &&
 	ply[i].dynamite_cnt < 5)
-      player_is_dropping = TRUE;
+      any_player_dropping = TRUE;
 
   CheckSingleStepMode_EM(action, frame, game_em.any_player_moving,
-			 player_is_dropping);
+			 game_em.any_player_snapping, any_player_dropping);
 
-#if 1
-  game_animscreen();
-
-#if 1
-#if 0
-  SyncDisplay();
-#endif
-
-  blitscreen();
-#endif
-#endif
+  RedrawPlayfield_EM(FALSE);
 }
 
 /* read input device for players */
@@ -158,6 +119,9 @@ void readjoy(byte action, struct PLAYER *ply)
   int north = 0, east = 0, south = 0, west = 0;
   int snap = 0, drop = 0;
 
+  if (game_em.use_single_button && action & (JOY_BUTTON_1 | JOY_BUTTON_2))
+    action |= JOY_BUTTON_1 | JOY_BUTTON_2;
+
   if (action & JOY_LEFT)
     west = 1;
 
@@ -176,16 +140,27 @@ void readjoy(byte action, struct PLAYER *ply)
   if (action & JOY_BUTTON_2)
     drop = 1;
 
-  ply->joy_snap = snap;
+  /* always update drop action */
   ply->joy_drop = drop;
 
-  if (ply->joy_stick || (north | east | south | west))
+  if (ply->joy_stick || (north | east | south | west))	/* (no "| snap"!) */
   {
     ply->joy_n = north;
     ply->joy_e = east;
     ply->joy_s = south;
     ply->joy_w = west;
+
+    /* when storing last action, only update snap action with direction */
+    /* (prevents clearing direction if snapping stopped before frame 7) */
+    ply->joy_snap = snap;
   }
+
+  /* if no direction was stored before, allow setting snap to current state */
+  if (!ply->joy_n &&
+      !ply->joy_e &&
+      !ply->joy_s &&
+      !ply->joy_w)
+    ply->joy_snap = snap;
 }
 
 void SaveEngineSnapshotValues_EM()
