@@ -14,14 +14,4 @@ extern int sound_length[SAMPLE_MAX];
 
 #define MIXER_MAX 4	/* maximum number of samples we can play at once */
 
-#if defined(AUDIO_UNIX_NATIVE)
-
-enum
-{
-  AUDIO_ULAW = 0,
-  AUDIO_U8
-};
-
-#endif	/* AUDIO_UNIX_NATIVE */
-
 #endif	/* SAMPLE_H */
