@@ -7,13 +7,8 @@
 
 #include "global.h"
 
-#if 1
 extern int AnimationPosTable[SP_MAX_PLAYFIELD_SIZE];
 extern byte AnimationSubTable[SP_MAX_PLAYFIELD_SIZE];
-#else
-extern byte *AnimationSubTable;
-extern int *AnimationPosTable;
-#endif
 
 extern void subDoGameStuff();
 
