@@ -1,15 +1,13 @@
-/***********************************************************
-* Artsoft Retro-Game Library                               *
-*----------------------------------------------------------*
-* (c) 1994-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* gadgets.c                                                *
-***********************************************************/
+// ============================================================================
+// Artsoft Retro-Game Library
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// gadgets.c
+// ============================================================================
 
 #include <stdarg.h>
 #include <string.h>
@@ -26,6 +24,14 @@
 #define DG_BUFFERED		0
 #define DG_DIRECT		1
 
+#define OPTION_TEXT_SELECTABLE(g, t)					\
+  (t[0] != g->selectbox.char_unselectable &&				\
+   t[0] != '\0' &&							\
+   !strEqual(t, " "))
+#define CURRENT_OPTION_SELECTABLE(g)					\
+  OPTION_TEXT_SELECTABLE(g, g->selectbox.options[g->selectbox.current_index].text)
+
+
 static struct GadgetInfo *gadget_list_first_entry = NULL;
 static struct GadgetInfo *gadget_list_last_entry = NULL;
 static struct GadgetInfo *last_info_gi = NULL;
@@ -88,6 +94,16 @@ static struct GadgetInfo *getGadgetInfoFromMousePosition(int mx, int my,
     boolean check_horizontal = (IS_WHEEL_BUTTON_HORIZONTAL(button) ||
 				GetKeyModState() & KMOD_Shift);
 
+    /* check for the first active scrollbar directly under the mouse pointer */
+    for (gi = gadget_list_first_entry; gi != NULL; gi = gi->next)
+    {
+      if (gi->mapped && gi->active &&
+	  (gi->type & GD_TYPE_SCROLLBAR) &&
+	  mx >= gi->x && mx < gi->x + gi->width &&
+	  my >= gi->y && my < gi->y + gi->height)
+	return gi;
+    }
+
     /* check for the first active scrollbar with matching mouse wheel area */
     for (gi = gadget_list_first_entry; gi != NULL; gi = gi->next)
     {
@@ -202,7 +218,7 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
   int state = (pressed ? GD_BUTTON_PRESSED : GD_BUTTON_UNPRESSED);
   boolean redraw_selectbox = FALSE;
 
-  if (gi == NULL)
+  if (gi == NULL || gi->deactivated)
     return;
 
   gd = (!gi->active ? &gi->alt_design[state] :
@@ -213,15 +229,31 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
     case GD_TYPE_NORMAL_BUTTON:
     case GD_TYPE_CHECK_BUTTON:
     case GD_TYPE_RADIO_BUTTON:
+
       BlitBitmapOnBackground(gd->bitmap, drawto,
 			     gd->x, gd->y, gi->width, gi->height,
 			     gi->x, gi->y);
+
       if (gi->deco.design.bitmap)
-	BlitBitmap(gi->deco.design.bitmap, drawto,
-		   gi->deco.design.x, gi->deco.design.y,
-		   gi->deco.width, gi->deco.height,
-		   gi->x + gi->deco.x + (pressed ? gi->deco.xshift : 0),
-		   gi->y + gi->deco.y + (pressed ? gi->deco.yshift : 0));
+      {
+	// make sure that decoration does not overlap gadget border
+	int deco_x = gi->deco.x + (pressed ? gi->deco.xshift : 0);
+	int deco_y = gi->deco.y + (pressed ? gi->deco.yshift : 0);
+	int deco_width  = MIN(gi->deco.width,  gi->width  - deco_x);
+	int deco_height = MIN(gi->deco.height, gi->height - deco_y);
+
+	if (gi->deco.masked)
+	  BlitBitmapMasked(gi->deco.design.bitmap, drawto,
+			   gi->deco.design.x, gi->deco.design.y,
+			   deco_width, deco_height,
+			   gi->x + deco_x, gi->y + deco_y);
+	else
+	  BlitBitmap(gi->deco.design.bitmap, drawto,
+		     gi->deco.design.x, gi->deco.design.y,
+		     deco_width, deco_height,
+		     gi->x + deco_x, gi->y + deco_y);
+      }
+
       break;
 
     case GD_TYPE_TEXT_BUTTON:
@@ -381,16 +413,9 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 				   gi->height - 2 * border_y);
 
 	/* gadget text value */
-#if 1
 	DrawTextBuffer(gi->x + border_x, gi->y + border_y, gi->textarea.value,
 		       font_nr, gi->textarea.xsize, -1, gi->textarea.ysize, 0,
 		       BLIT_ON_BACKGROUND, FALSE, FALSE, FALSE);
-#else
-	DrawTextToTextArea(gi->x + border_x, gi->y + border_y,
-			   gi->textarea.value, font_nr, gi->textarea.xsize,
-			   gi->textarea.xsize, gi->textarea.ysize,
-			   BLIT_ON_BACKGROUND);
-#endif
 
 	cursor_letter = gi->textarea.value[gi->textarea.cursor_position];
 	cursor_string[0] = (cursor_letter != '\0' ? cursor_letter : ' ');
@@ -410,7 +435,8 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
       {
 	int i;
 	char text[MAX_GADGET_TEXTSIZE + 1];
-	int font_nr = (pressed ? gi->font_active : gi->font);
+	int font_nr_default = (pressed ? gi->font_active : gi->font);
+	int font_nr = font_nr_default;
 	int font_width = getFontWidth(font_nr);
 	int font_height = getFontHeight(font_nr);
 	int border_x = gi->border.xsize;
@@ -447,6 +473,10 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 		gi->selectbox.size);
 	text[gi->selectbox.size] = '\0';
 
+	/* set font value */
+	font_nr = (OPTION_TEXT_SELECTABLE(gi, text) ? font_nr_default :
+		   gi->font_unselectable);
+
 	/* gadget text value */
 	DrawTextExt(drawto, gi->x + border_x, gi->y + border_y, text,
 		    font_nr, BLIT_MASKED);
@@ -555,7 +585,11 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 	    strncpy(text, gi->selectbox.options[i].text, gi->selectbox.size);
 	    text[gi->selectbox.size] = '\0';
 
-	    if (i == gi->selectbox.current_index)
+	    font_nr = (OPTION_TEXT_SELECTABLE(gi, text) ? font_nr_default :
+		       gi->font_unselectable);
+
+	    if (i == gi->selectbox.current_index &&
+		OPTION_TEXT_SELECTABLE(gi, text))
 	    {
 	      FillRectangle(drawto,
 			    gi->selectbox.x + border_x,
@@ -694,6 +728,10 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
       return;
   }
 
+  // do not use direct gadget drawing anymore; this worked as a speed-up once,
+  // but would slow things down a lot now the screen is always fully redrawn
+  direct = FALSE;
+
   if (direct)
   {
     BlitBitmap(drawto, window,
@@ -706,9 +744,15 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 		 gi->selectbox.x,     gi->selectbox.y);
   }
   else
-    redraw_mask |= (gi->x < gfx.sx + gfx.sxsize ? REDRAW_FIELD :
-		    gi->y < gfx.dy + gfx.dysize ? REDRAW_DOOR_1 :
-		    gi->y > gfx.vy ? REDRAW_DOOR_2 : REDRAW_DOOR_3);
+  {
+    int x = gi->x;
+    int y = gi->y;
+
+    redraw_mask |= (IN_GFX_FIELD_FULL(x, y) ? REDRAW_FIELD :
+		    IN_GFX_DOOR_1(x, y) ? REDRAW_DOOR_1 :
+		    IN_GFX_DOOR_2(x, y) ? REDRAW_DOOR_2 :
+		    IN_GFX_DOOR_3(x, y) ? REDRAW_DOOR_3 : REDRAW_ALL);
+  }
 }
 
 static int get_minimal_size_for_numeric_input(int minmax_value)
@@ -731,7 +775,7 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 {
   int tag = first_tag;
 
-  if (gi == NULL)
+  if (gi == NULL || gi->deactivated)
     return;
 
   while (tag != GDI_END)
@@ -873,12 +917,18 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->font = va_arg(ap, int);
 	if (gi->font_active == 0)
 	  gi->font_active = gi->font;
+	if (gi->font_unselectable == 0)
+	  gi->font_unselectable = gi->font;
 	break;
 
       case GDI_TEXT_FONT_ACTIVE:
 	gi->font_active = va_arg(ap, int);
 	break;
 
+      case GDI_TEXT_FONT_UNSELECTABLE:
+	gi->font_unselectable = va_arg(ap, int);
+	break;
+
       case GDI_SELECTBOX_OPTIONS:
 	gi->selectbox.options = va_arg(ap, struct ValueTextInfo *);
 	break;
@@ -887,6 +937,10 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->selectbox.index = va_arg(ap, int);
 	break;
 
+      case GDI_SELECTBOX_CHAR_UNSELECTABLE:
+	gi->selectbox.char_unselectable = (char)va_arg(ap, int);
+	break;
+
       case GDI_DESIGN_UNPRESSED:
 	gi->design[GD_BUTTON_UNPRESSED].bitmap = va_arg(ap, Bitmap *);
 	gi->design[GD_BUTTON_UNPRESSED].x = va_arg(ap, int);
@@ -945,6 +999,10 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->deco.yshift = va_arg(ap, int);
 	break;
 
+      case GDI_DECORATION_MASKED:
+	gi->deco.masked = (boolean)va_arg(ap, int);
+	break;
+
       case GDI_EVENT_MASK:
 	gi->event_mask = va_arg(ap, unsigned int);
 	break;
@@ -954,14 +1012,12 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->drawing.area_ysize = va_arg(ap, int);
 
 	/* determine dependent values for drawing area gadget, if needed */
-	if (gi->width == 0 && gi->height == 0 &&
-	    gi->drawing.item_xsize !=0 && gi->drawing.item_ysize !=0)
+	if (gi->drawing.item_xsize != 0 && gi->drawing.item_ysize != 0)
 	{
 	  gi->width = gi->drawing.area_xsize * gi->drawing.item_xsize;
 	  gi->height = gi->drawing.area_ysize * gi->drawing.item_ysize;
 	}
-	else if (gi->drawing.item_xsize == 0 && gi->drawing.item_ysize == 0 &&
-		 gi->width != 0 && gi->height != 0)
+	else if (gi->width != 0 && gi->height != 0)
 	{
 	  gi->drawing.item_xsize = gi->width / gi->drawing.area_xsize;
 	  gi->drawing.item_ysize = gi->height / gi->drawing.area_ysize;
@@ -984,14 +1040,12 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->drawing.item_ysize = va_arg(ap, int);
 
 	/* determine dependent values for drawing area gadget, if needed */
-	if (gi->width == 0 && gi->height == 0 &&
-	    gi->drawing.area_xsize !=0 && gi->drawing.area_ysize !=0)
+	if (gi->drawing.area_xsize != 0 && gi->drawing.area_ysize != 0)
 	{
 	  gi->width = gi->drawing.area_xsize * gi->drawing.item_xsize;
 	  gi->height = gi->drawing.area_ysize * gi->drawing.item_ysize;
 	}
-	else if (gi->drawing.area_xsize == 0 && gi->drawing.area_ysize == 0 &&
-		 gi->width != 0 && gi->height != 0)
+	else if (gi->width != 0 && gi->height != 0)
 	{
 	  gi->drawing.area_xsize = gi->width / gi->drawing.item_xsize;
 	  gi->drawing.area_ysize = gi->height / gi->drawing.item_ysize;
@@ -1041,11 +1095,17 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
     tag = va_arg(ap, int);	/* read next tag */
   }
 
-  /* check if gadget is complete */
+  gi->deactivated = FALSE;
+
+  /* check if gadget has undefined bitmaps */
   if (gi->type != GD_TYPE_DRAWING_AREA &&
-      (!gi->design[GD_BUTTON_UNPRESSED].bitmap ||
-       !gi->design[GD_BUTTON_PRESSED].bitmap))
-    Error(ERR_EXIT, "gadget incomplete (missing Bitmap)");
+      (gi->design[GD_BUTTON_UNPRESSED].bitmap == NULL ||
+       gi->design[GD_BUTTON_PRESSED].bitmap == NULL))
+    gi->deactivated = TRUE;
+
+  /* check if gadget is placed off-screen */
+  if (gi->x < 0 || gi->y < 0)
+    gi->deactivated = TRUE;
 
   /* adjust gadget values in relation to other gadget values */
 
@@ -1207,7 +1267,7 @@ void ModifyGadget(struct GadgetInfo *gi, int first_tag, ...)
 
 void RedrawGadget(struct GadgetInfo *gi)
 {
-  if (gi == NULL)
+  if (gi == NULL || gi->deactivated)
     return;
 
   if (gi->mapped)
@@ -1248,6 +1308,9 @@ void FreeGadget(struct GadgetInfo *gi)
 {
   struct GadgetInfo *gi_previous = gadget_list_first_entry;
 
+  if (gi == NULL)
+    return;
+
   /* prevent "last_info_gi" from pointing to memory that will be freed */
   if (last_info_gi == gi)
     last_info_gi = NULL;
@@ -1292,7 +1355,7 @@ static struct GadgetInfo *last_gi = NULL;
 
 static void MapGadgetExt(struct GadgetInfo *gi, boolean redraw)
 {
-  if (gi == NULL || gi->mapped)
+  if (gi == NULL || gi->deactivated || gi->mapped)
     return;
 
   gi->mapped = TRUE;
@@ -1308,7 +1371,7 @@ void MapGadget(struct GadgetInfo *gi)
 
 void UnmapGadget(struct GadgetInfo *gi)
 {
-  if (gi == NULL || !gi->mapped)
+  if (gi == NULL || gi->deactivated || !gi->mapped)
     return;
 
   gi->mapped = FALSE;
@@ -1324,9 +1387,11 @@ void UnmapGadget(struct GadgetInfo *gi)
 #define MULTIMAP_PLAYFIELD	(1 << 3)
 #define MULTIMAP_DOOR_1		(1 << 4)
 #define MULTIMAP_DOOR_2		(1 << 5)
+#define MULTIMAP_DOOR_3		(1 << 6)
 #define MULTIMAP_ALL		(MULTIMAP_PLAYFIELD | \
-				 MULTIMAP_DOOR_1 | \
-				 MULTIMAP_DOOR_2)
+				 MULTIMAP_DOOR_1    | \
+				 MULTIMAP_DOOR_2    | \
+				 MULTIMAP_DOOR_3)
 
 static void MultiMapGadgets(int mode)
 {
@@ -1336,12 +1401,13 @@ static void MultiMapGadgets(int mode)
 
   while (gi != NULL)
   {
-    if ((mode & MULTIMAP_PLAYFIELD &&
-	 gi->x < gfx.sx + gfx.sxsize) ||
-	(mode & MULTIMAP_DOOR_1 &&
-	 gi->x >= gfx.dx && gi->y < gfx.dy + gfx.dysize) ||
-	(mode & MULTIMAP_DOOR_2 &&
-	 gi->x >= gfx.dx && gi->y > gfx.dy + gfx.dysize) ||
+    int x = gi->x;
+    int y = gi->y;
+
+    if ((mode & MULTIMAP_PLAYFIELD && IN_GFX_FIELD_FULL(x, y)) ||
+	(mode & MULTIMAP_DOOR_1 && IN_GFX_DOOR_1(x, y)) ||
+	(mode & MULTIMAP_DOOR_2 && IN_GFX_DOOR_2(x, y)) ||
+	(mode & MULTIMAP_DOOR_3 && IN_GFX_DOOR_3(x, y)) ||
 	(mode & MULTIMAP_ALL) == MULTIMAP_ALL)
     {
       if (mode & MULTIMAP_UNMAP)
@@ -1458,9 +1524,6 @@ boolean HandleGadgets(int mx, int my, int button)
   boolean gadget_dragging;
   boolean gadget_released;
   boolean gadget_released_inside;
-#if 0
-  boolean gadget_released_inside_select_line;
-#endif
   boolean gadget_released_inside_select_area;
   boolean gadget_released_off_borders;
   boolean changed_position = FALSE;
@@ -1542,6 +1605,8 @@ boolean HandleGadgets(int mx, int my, int button)
 
     if (gadget_pressed_inside_select_line)
       new_gi = NULL;
+
+    StopTextInput();
   }
 
   gadget_pressed =
@@ -1562,19 +1627,9 @@ boolean HandleGadgets(int mx, int my, int button)
 
   /* when handling selectbox, set additional state values */
   if (gadget_released_inside && (last_gi->type & GD_TYPE_SELECTBOX))
-  {
-#if 0
-    gadget_released_inside_select_line = insideSelectboxLine(last_gi, mx, my);
-#endif
     gadget_released_inside_select_area = insideSelectboxArea(last_gi, mx, my);
-  }
   else
-  {
-#if 0
-    gadget_released_inside_select_line = FALSE;
-#endif
     gadget_released_inside_select_area = FALSE;
-  }
 
   /* setting state for handling over-large selectbox */
   if (keep_selectbox_open && (press_event || !mouse_inside_select_line))
@@ -1598,7 +1653,8 @@ boolean HandleGadgets(int mx, int my, int button)
     if (gi->type & GD_TYPE_SELECTBOX &&
 	(keep_selectbox_open ||
 	 mouse_released_where_pressed ||
-	 !gadget_released_inside_select_area))	     /* selectbox stays open */
+	 !gadget_released_inside_select_area ||
+	 !CURRENT_OPTION_SELECTABLE(gi)))	    /* selectbox stays open */
     {
       gi->selectbox.stay_open = TRUE;
       pressed_mx = 0;
@@ -1652,6 +1708,9 @@ boolean HandleGadgets(int mx, int my, int button)
 
       if (gi->textinput.cursor_position != old_cursor_position)
 	DrawGadget(gi, DG_PRESSED, gi->direct_draw);
+
+      if (press_event)
+	StartTextInput(gi->x, gi->y, gi->width, gi->height);
     }
     else if (gi->type & GD_TYPE_TEXT_AREA && button != 0 && !motion_status)
     {
@@ -1666,6 +1725,9 @@ boolean HandleGadgets(int mx, int my, int button)
 
       if (gi->textarea.cursor_position != old_cursor_position)
 	DrawGadget(gi, DG_PRESSED, gi->direct_draw);
+
+      if (press_event)
+	StartTextInput(gi->x, gi->y, gi->width, gi->height);
     }
     else if (gi->type & GD_TYPE_SELECTBOX && gi->selectbox.open &&
 	     !keep_selectbox_open)
@@ -1696,7 +1758,7 @@ boolean HandleGadgets(int mx, int my, int button)
       new_gi->event.type = GD_EVENT_INFO_ENTERING;
       new_gi->callback_info(new_gi);
     }
-    else if (last_info_gi != NULL)
+    else if (last_info_gi != NULL && last_info_gi->mapped)
     {
       last_info_gi->event.type = GD_EVENT_INFO_LEAVING;
       last_info_gi->callback_info(last_info_gi);
@@ -1759,7 +1821,7 @@ boolean HandleGadgets(int mx, int my, int button)
 	{
 	  boolean scroll_single_step = ((GetKeyModState() & KMOD_Alt) != 0);
 
-	  item_steps = (scroll_single_step ? 1 : DEFAULT_WHEEL_STEPS);
+	  item_steps = (scroll_single_step ? 1 : wheel_steps);
 	  item_direction = (button == MB_WHEEL_UP ||
 			    button == MB_WHEEL_LEFT ? -1 : +1);
 	}
@@ -1956,7 +2018,8 @@ boolean HandleGadgets(int mx, int my, int button)
     {
       if (keep_selectbox_open ||
 	  mouse_released_where_pressed ||
-	  !gadget_released_inside_select_area)	     /* selectbox stays open */
+	  !gadget_released_inside_select_area ||
+	  !CURRENT_OPTION_SELECTABLE(gi))	    /* selectbox stays open */
       {
 	deactivate_gadget = FALSE;
 	gadget_changed = FALSE;
@@ -2021,13 +2084,17 @@ boolean HandleGadgetsKeyInput(Key key)
 {
   struct GadgetInfo *gi = last_gi;
 
-  if (gi == NULL || !gi->mapped ||
+  if (gi == NULL || gi->deactivated || !gi->mapped ||
       !(gi->type & GD_TYPE_TEXT_INPUT ||
 	gi->type & GD_TYPE_TEXT_AREA ||
 	gi->type & GD_TYPE_SELECTBOX))
     return FALSE;
 
-  if (key == KSYM_Return)	/* valid for both text input and selectbox */
+  if (key == KSYM_Escape)
+  {
+    StopTextInput();
+  }
+  else if (key == KSYM_Return)	/* valid for both text input and selectbox */
   {
     boolean gadget_changed = ((gi->event_mask & GD_EVENT_TEXT_RETURN) != 0);
 
@@ -2039,6 +2106,8 @@ boolean HandleGadgetsKeyInput(Key key)
 	strcpy(gi->textinput.last_value, gi->textinput.value);
       else
 	gadget_changed = FALSE;
+
+      StopTextInput();
     }
     else if (gi->type & GD_TYPE_SELECTBOX)
     {
