@@ -1,15 +1,13 @@
-/***********************************************************
-* Artsoft Retro-Game Library                               *
-*----------------------------------------------------------*
-* (c) 1994-2002 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* sound.h                                                  *
-***********************************************************/
+// ============================================================================
+// Artsoft Retro-Game Library
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// sound.h
+// ============================================================================
 
 #ifndef SOUND_H
 #define SOUND_H
@@ -17,20 +15,7 @@
 #include "system.h"
 
 
-#if defined(PLATFORM_UNIX) && !defined(TARGET_SDL)
-#define AUDIO_UNIX_NATIVE
-#endif
-
-#if defined(PLATFORM_LINUX) || defined(PLATFORM_FREEBSD) || defined(VOXWARE)
-#define AUDIO_LINUX_IOCTL
-#endif
-
-#if defined(AUDIO_LINUX_IOCTL) || defined(PLATFORM_NETBSD)
-#define AUDIO_STREAMING_DSP
-#endif
-
 /* values for platform specific sound initialization */
-#define AUDIO_SAMPLE_RATE_8000		8000
 #define AUDIO_SAMPLE_RATE_22050		22050
 
 #define AUDIO_FRAGMENT_SIZE_512		512
@@ -48,25 +33,15 @@
 #define AUDIO_FORMAT_LE			(1 << 2)
 #define AUDIO_FORMAT_BE			(1 << 3)
 
-#if defined(AUDIO_UNIX_NATIVE) && !defined(AUDIO_STREAMING_DSP)
-#define DEFAULT_AUDIO_SAMPLE_RATE	AUDIO_SAMPLE_RATE_8000
-#else
 #define DEFAULT_AUDIO_SAMPLE_RATE	AUDIO_SAMPLE_RATE_22050
-#endif
 
-#if defined(PLATFORM_HPUX)
-#define DEFAULT_AUDIO_FRAGMENT_SIZE	AUDIO_FRAGMENT_SIZE_32768
-#elif defined(PLATFORM_WIN32)
+#if defined(PLATFORM_WIN32)
 #define DEFAULT_AUDIO_FRAGMENT_SIZE	AUDIO_FRAGMENT_SIZE_1024
 #else
 #define DEFAULT_AUDIO_FRAGMENT_SIZE	AUDIO_FRAGMENT_SIZE_512
 #endif
 
-#if defined(TARGET_SDL)
 #define NUM_MIXER_CHANNELS		MIX_CHANNELS
-#else
-#define NUM_MIXER_CHANNELS		8
-#endif
 
 #define MUSIC_CHANNEL			0
 #define FIRST_SOUND_CHANNEL		1
@@ -81,6 +56,7 @@
 #define SND_CTRL_ALL_SOUNDS		(1 << 4)
 #define SND_CTRL_RELOAD_SOUNDS		(1 << 5)
 #define SND_CTRL_RELOAD_MUSIC		(1 << 6)
+#define SND_CTRL_EXPIRE_LOOPS		(1 << 7)
 
 #define SND_CTRL_PLAY_SOUND		(SND_CTRL_NONE)
 #define SND_CTRL_PLAY_LOOP		(SND_CTRL_LOOP)
@@ -101,19 +77,14 @@
 #define IS_RELOADING(x)			((x).state & (SND_CTRL_RELOAD_SOUNDS |\
 						      SND_CTRL_RELOAD_MUSIC))
 #define ALL_SOUNDS(x)			((x).state & SND_CTRL_ALL_SOUNDS)
+#define SET_EXPIRE_LOOPS(x)		((x).state & SND_CTRL_EXPIRE_LOOPS)
 
 #define MAP_NOCONF_MUSIC(x)		(-((x) + 1))
 #define UNMAP_NOCONF_MUSIC(x)		MAP_NOCONF_MUSIC(x)
 
 
 #define SOUND_MIN_VOLUME		0
-#if defined(TARGET_SDL)
 #define SOUND_MAX_VOLUME		SDL_MIX_MAXVOLUME
-#elif defined(TARGET_ALLEGRO)
-#define SOUND_MAX_VOLUME		255
-#else
-#define SOUND_MAX_VOLUME		128
-#endif
 
 #define SOUND_MAX_LEFT			0
 #define SOUND_MAX_RIGHT			255
@@ -144,11 +115,13 @@ void StopMusic(void);
 void StopSound(int);
 void StopSounds(void);
 void StopSoundExt(int, int);
+void ExpireSoundLoops(boolean);
 
 int getSoundListSize();
 int getMusicListSize();
 struct FileInfo *getSoundListEntry(int);
 struct FileInfo *getMusicListEntry(int);
+char *getCurrentlyPlayingMusicFilename();
 int getSoundListPropertyMappingSize();
 int getMusicListPropertyMappingSize();
 struct PropertyMapping *getSoundListPropertyMapping();
