@@ -324,6 +324,19 @@
 
 #define STYLE_DEFAULT		STYLE_NONE
 
+/* values for special global animation events */
+#define ANIM_EVENT_NONE		0
+#define ANIM_EVENT_SELF		(1 << 16)
+#define ANIM_EVENT_ANY		(1 << 17)
+
+#define ANIM_EVENT_ANIM_BIT	0
+#define ANIM_EVENT_PART_BIT	8
+
+#define ANIM_EVENT_ANIM_MASK	(0xff << ANIM_EVENT_ANIM_BIT)
+#define ANIM_EVENT_PART_MASK	(0xff << ANIM_EVENT_PART_BIT)
+
+#define ANIM_EVENT_DEFAULT	ANIM_EVENT_NONE
+
 /* values for fade mode */
 #define FADE_TYPE_NONE		0
 #define FADE_TYPE_FADE_IN	(1 << 0)
@@ -357,6 +370,7 @@
 #define POS_LOWER		5
 #define POS_BOTTOM		6
 #define POS_ANY			7
+#define POS_LAST		8
 
 /* values for text alignment */
 #define ALIGN_LEFT		(1 << 0)
@@ -428,6 +442,10 @@
 /* maximum number of levels in a level set */
 #define MAX_LEVELS		1000
 
+/* maximum number of global animation and parts */
+#define MAX_GLOBAL_ANIMS		32
+#define MAX_GLOBAL_ANIM_PARTS		32
+
 /* default name for empty highscore entry */
 #define EMPTY_PLAYER_NAME	"no name"
 
@@ -480,6 +498,7 @@
 #define SCORES_DIRECTORY	"scores"
 #define DOCS_DIRECTORY		"docs"
 #define CACHE_DIRECTORY		"cache"
+#define CONF_DIRECTORY		"conf"
 
 #define GFX_CLASSIC_SUBDIR	"gfx_classic"
 #define SND_CLASSIC_SUBDIR	"snd_classic"
@@ -513,6 +532,8 @@
 #define TAPEFILE_EXTENSION	"tape"
 #define SCOREFILE_EXTENSION	"score"
 
+#define GAMECONTROLLER_BASENAME	"gamecontrollerdb.txt"
+
 #define LOG_OUT_BASENAME	"stdout.txt"
 #define LOG_ERR_BASENAME	"stderr.txt"
 
@@ -780,6 +801,7 @@ struct OptionInfo
   char *sounds_directory;
   char *music_directory;
   char *docs_directory;
+  char *conf_directory;
 
   char *execute_command;
 
@@ -921,13 +943,14 @@ struct GfxInfo
 
 struct OverlayInfo
 {
-  boolean active;
+  boolean enabled;		/* overlay generally enabled or disabled */
+  boolean active;		/* overlay activated (depending on game mode) */
 };
 
 struct JoystickInfo
 {
   int status;
-  int fd[MAX_PLAYERS];		/* file descriptor of player's joystick */
+  int nr[MAX_PLAYERS];		/* joystick number for each player */
 };
 
 struct SetupJoystickInfo
@@ -1449,6 +1472,7 @@ void InitGfxDrawGlobalBorderFunction(void (*draw_global_border_function)(int));
 void InitGfxCustomArtworkInfo();
 void InitGfxOtherSettings();
 void InitOverlayInfo();
+void SetOverlayEnabled(boolean);
 void SetOverlayActive(boolean);
 boolean GetOverlayActive();
 void SetDrawDeactivationMask(int);
@@ -1526,5 +1550,7 @@ boolean CheckCloseWindowEvent(ClientMessageEvent *);
 
 void InitJoysticks();
 boolean ReadJoystick(int, int *, int *, boolean *, boolean *);
+boolean CheckJoystickOpened(int);
+void ClearJoystickState();
 
 #endif /* SYSTEM_H */
