@@ -1,15 +1,13 @@
-/***********************************************************
-* Artsoft Retro-Game Library                               *
-*----------------------------------------------------------*
-* (c) 1994-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* text.c                                                   *
-***********************************************************/
+// ============================================================================
+// Artsoft Retro-Game Library
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// text.c
+// ============================================================================
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -22,90 +20,6 @@
 /* font functions                                                            */
 /* ========================================================================= */
 
-#if defined(TARGET_X11_NATIVE_PERFORMANCE_WORKAROUND)
-static GC	font_clip_gc = None;
-
-static void InitFontClipmasks()
-{
-  XGCValues clip_gc_values;
-  unsigned int clip_gc_valuemask;
-  GC copy_clipmask_gc;
-  int i, j;
-
-  /* This stuff is needed because X11 (XSetClipOrigin(), to be precise) is
-     often very slow when preparing a masked XCopyArea() for big Pixmaps.
-     To prevent this, create small (tile-sized) mask Pixmaps which will then
-     be set much faster with XSetClipOrigin() and speed things up a lot. */
-
-  clip_gc_values.graphics_exposures = False;
-  clip_gc_valuemask = GCGraphicsExposures;
-  font_clip_gc = XCreateGC(display, window->drawable,
-			   clip_gc_valuemask, &clip_gc_values);
-
-  /* create graphic context structures needed for clipping */
-  clip_gc_values.graphics_exposures = False;
-  clip_gc_valuemask = GCGraphicsExposures;
-  copy_clipmask_gc = XCreateGC(display,
-			       gfx.font_bitmap_info[0].bitmap->clip_mask,
-			       clip_gc_valuemask, &clip_gc_values);
-
-  /* create only those clipping Pixmaps we really need */
-  for (i = 0; i < gfx.num_fonts; i++)
-  {
-    if (gfx.font_bitmap_info[i].bitmap == NULL)
-      continue;
-
-    gfx.font_bitmap_info[i].clip_mask =
-      checked_calloc(gfx.font_bitmap_info[i].num_chars * sizeof(Pixmap));
-
-    for (j = 0; j < gfx.font_bitmap_info[i].num_chars; j++)
-    {
-      Bitmap *src_bitmap = gfx.font_bitmap_info[i].bitmap;
-      Pixmap src_pixmap = src_bitmap->clip_mask;
-      int xpos = j % gfx.font_bitmap_info[i].num_chars_per_line;
-      int ypos = j / gfx.font_bitmap_info[i].num_chars_per_line;
-      int width  = gfx.font_bitmap_info[i].width;
-      int height = gfx.font_bitmap_info[i].height;
-      int src_x = gfx.font_bitmap_info[i].src_x + xpos * width;
-      int src_y = gfx.font_bitmap_info[i].src_y + ypos * height;
-
-      gfx.font_bitmap_info[i].clip_mask[j] =
-	XCreatePixmap(display, window->drawable, width, height, 1);
-
-      XCopyArea(display, src_pixmap, gfx.font_bitmap_info[i].clip_mask[j],
-		copy_clipmask_gc, src_x, src_y, width, height, 0, 0);
-    }
-  }
-
-  XFreeGC(display, copy_clipmask_gc);
-}
-
-static void FreeFontClipmasks()
-{
-  int i, j;
-
-  if (gfx.num_fonts == 0 || gfx.font_bitmap_info[0].bitmap == NULL)
-    return;
-
-  for (i = 0; i < gfx.num_fonts; i++)
-  {
-    if (gfx.font_bitmap_info[i].clip_mask)
-    {
-      for (j = 0; j < gfx.font_bitmap_info[i].num_chars; j++)
-	XFreePixmap(display, gfx.font_bitmap_info[i].clip_mask[j]);
-      free(gfx.font_bitmap_info[i].clip_mask);
-    }
-
-    gfx.font_bitmap_info[i].clip_mask = NULL;
-    gfx.font_bitmap_info[i].num_chars = 0;
-  }
-
-  if (font_clip_gc)
-    XFreeGC(display, font_clip_gc);
-  font_clip_gc = None;
-}
-#endif /* TARGET_X11_NATIVE_PERFORMANCE_WORKAROUND */
-
 void InitFontInfo(struct FontBitmapInfo *font_bitmap_info, int num_fonts,
 		  int (*select_font_function)(int),
 		  int (*get_font_from_token_function)(char *))
@@ -114,10 +28,6 @@ void InitFontInfo(struct FontBitmapInfo *font_bitmap_info, int num_fonts,
   gfx.font_bitmap_info = font_bitmap_info;
   gfx.select_font_function = select_font_function;
   gfx.get_font_from_token_function = get_font_from_token_function;
-
-#if defined(TARGET_X11_NATIVE_PERFORMANCE_WORKAROUND)
-  InitFontClipmasks();
-#endif
 }
 
 void FreeFontInfo(struct FontBitmapInfo *font_bitmap_info)
@@ -125,10 +35,6 @@ void FreeFontInfo(struct FontBitmapInfo *font_bitmap_info)
   if (font_bitmap_info == NULL)
     return;
 
-#if defined(TARGET_X11_NATIVE_PERFORMANCE_WORKAROUND)
-  FreeFontClipmasks();
-#endif
-
   free(font_bitmap_info);
 }
 
@@ -153,6 +59,20 @@ int getFontHeight(int font_nr)
   return gfx.font_bitmap_info[font_bitmap_id].height;
 }
 
+int getFontDrawOffsetX(int font_nr)
+{
+  int font_bitmap_id = gfx.select_font_function(font_nr);
+
+  return gfx.font_bitmap_info[font_bitmap_id].draw_xoffset;
+}
+
+int getFontDrawOffsetY(int font_nr)
+{
+  int font_bitmap_id = gfx.select_font_function(font_nr);
+
+  return gfx.font_bitmap_info[font_bitmap_id].draw_yoffset;
+}
+
 int getTextWidth(char *text, int font_nr)
 {
   return (text != NULL ? strlen(text) * getFontWidth(font_nr) : 0);
@@ -195,10 +115,14 @@ void getFontCharSource(int font_nr, char c, Bitmap **bitmap, int *x, int *y)
 int maxWordLengthInString(char *text)
 {
   char *text_ptr;
-  int max_word_len = 0;
+  int word_len = 0, max_word_len = 0;
 
   for (text_ptr = text; *text_ptr; text_ptr++)
-    max_word_len = (*text_ptr != ' ' ? max_word_len + 1 : 0);
+  {
+    word_len = (*text_ptr != ' ' ? word_len + 1 : 0);
+
+    max_word_len = MAX(word_len, max_word_len);
+  }
 
   return max_word_len;
 }
@@ -208,16 +132,12 @@ int maxWordLengthInString(char *text)
 /* simple text drawing functions                                             */
 /* ========================================================================= */
 
-void DrawInitTextExt(char *text, int ypos, int font_nr, boolean force)
+void DrawInitText(char *text, int ypos, int font_nr)
 {
-  static unsigned int progress_delay = 0;
-  unsigned int progress_delay_value = 100;	/* (in milliseconds) */
+  LimitScreenUpdates(TRUE);
 
   UPDATE_BUSY_STATE();
 
-  if (!force && !DelayReached(&progress_delay, progress_delay_value))
-    return;
-
   if (window != NULL &&
       gfx.draw_init_text &&
       gfx.num_fonts > 0 &&
@@ -231,21 +151,10 @@ void DrawInitTextExt(char *text, int ypos, int font_nr, boolean force)
     ClearRectangle(drawto, 0, y, width, height);
     DrawTextExt(drawto, x, y, text, font_nr, BLIT_OPAQUE);
 
-    /* this makes things significantly faster than directly drawing to window */
-    BlitBitmap(drawto, window, 0, y, width, height, 0, y);
+    BlitBitmap(drawto, window, 0, 0, video.width, video.height, 0, 0);
   }
 }
 
-void DrawInitText(char *text, int ypos, int font_nr)
-{
-  DrawInitTextExt(text, ypos, font_nr, TRUE);
-}
-
-void DrawInitTextIfNeeded(char *text, int ypos, int font_nr)
-{
-  DrawInitTextExt(text, ypos, font_nr, FALSE);
-}
-
 void DrawTextF(int x, int y, int font_nr, char *format, ...)
 {
   char buffer[MAX_OUTPUT_LINESIZE + 1];
@@ -288,11 +197,6 @@ void DrawTextSCentered(int y, int font_nr, char *text)
 	   gfx.sy + y, text, font_nr);
 }
 
-void DrawTextCentered(int y, int font_nr, char *text)
-{
-  DrawText((gfx.sxsize - getTextWidth(text, font_nr)) / 2, y, text, font_nr);
-}
-
 void DrawTextSAligned(int x, int y, char *text, int font_nr, int align)
 {
   DrawText(gfx.sx + ALIGNED_XPOS(x, getTextWidth(text, font_nr), align),
@@ -314,28 +218,24 @@ void DrawText(int x, int y, char *text, int font_nr)
 
   DrawTextExt(drawto, x, y, text, font_nr, mask_mode);
 
-  if (x < gfx.dx)
+  if (IN_GFX_FIELD_FULL(x, y))
     redraw_mask |= REDRAW_FIELD;
-  else if (y < gfx.vy || gfx.vy == 0)
+  else if (IN_GFX_DOOR_1(x, y))
     redraw_mask |= REDRAW_DOOR_1;
+  else if (IN_GFX_DOOR_2(x, y))
+    redraw_mask |= REDRAW_DOOR_2;
+  else if (IN_GFX_DOOR_3(x, y))
+    redraw_mask |= REDRAW_DOOR_3;
+  else
+    redraw_mask |= REDRAW_ALL;
 }
 
 void DrawTextExt(DrawBuffer *dst_bitmap, int dst_x, int dst_y, char *text,
 		 int font_nr, int mask_mode)
 {
-#if 1
   struct FontBitmapInfo *font = getFontBitmapInfo(font_nr);
-#else
-  int font_bitmap_id = gfx.select_font_function(font_nr);
-  struct FontBitmapInfo *font = &gfx.font_bitmap_info[font_bitmap_id];
-#endif
   int font_width = getFontWidth(font_nr);
   int font_height = getFontHeight(font_nr);
-#if 0
-  int border_1 = gfx.sx + gfx.sxsize;
-  int border_2 = gfx.dx + gfx.dxsize;
-  int dst_x_start = dst_x;
-#endif
   Bitmap *src_bitmap;
   int src_x, src_y;
   char *text_ptr = text;
@@ -369,15 +269,8 @@ void DrawTextExt(DrawBuffer *dst_bitmap, int dst_x, int dst_y, char *text,
     }
 
     /* clip text at the right side of the window */
-#if 1
     if (dst_x + font_width > video.width)
       break;
-#else
-    /* (this does not work well when trying to print text to whole screen) */
-    if ((dst_x_start < border_1 && dst_x + font_width > border_1) ||
-	(dst_x_start < border_2 && dst_x + font_width > border_2))
-      break;
-#endif
 
     if (mask_mode == BLIT_INVERSE)	/* special mode for text gadgets */
     {
@@ -394,17 +287,9 @@ void DrawTextExt(DrawBuffer *dst_bitmap, int dst_x, int dst_y, char *text,
 		   font_width, font_height, dst_x, dst_y);
       }
 
-#if defined(TARGET_SDL)
       /* second step: draw masked inverted character */
       SDLCopyInverseMasked(src_bitmap, dst_bitmap, src_x, src_y,
 			   font_width, font_height, dst_x, dst_y);
-#else
-      /* second step: draw masked black rectangle (use "space" character) */
-      SetClipOrigin(src_bitmap, src_bitmap->stored_clip_gc,
-		    dst_x - src_x, dst_y - src_y);
-      BlitBitmapMasked(src_bitmap, dst_bitmap, 0, 0,
-		       font_width, font_height, dst_x, dst_y);
-#endif
     }
     else if (mask_mode == BLIT_MASKED || mask_mode == BLIT_ON_BACKGROUND)
     {
@@ -415,19 +300,6 @@ void DrawTextExt(DrawBuffer *dst_bitmap, int dst_x, int dst_y, char *text,
 				   font_width, font_height);
       }
 
-#if defined(TARGET_X11_NATIVE_PERFORMANCE_WORKAROUND)
-      /* use special font tile clipmasks */
-      {
-	int font_pos = getFontCharPosition(font_nr, c);
-
-	SetClipMask(src_bitmap, font_clip_gc, font->clip_mask[font_pos]);
-	SetClipOrigin(src_bitmap, font_clip_gc, dst_x, dst_y);
-      }
-#else
-      SetClipOrigin(src_bitmap, src_bitmap->stored_clip_gc,
-		    dst_x - src_x, dst_y - src_y);
-#endif
-
       BlitBitmapMasked(src_bitmap, dst_bitmap, src_x, src_y,
 		       font_width, font_height, dst_x, dst_y);
     }
@@ -450,24 +322,24 @@ void DrawTextExt(DrawBuffer *dst_bitmap, int dst_x, int dst_y, char *text,
 
 char *GetTextBufferFromFile(char *filename, int max_lines)
 {
-  FILE *file;
+  File *file;
   char *buffer;
   int num_lines = 0;
 
   if (filename == NULL)
     return NULL;
 
-  if (!(file = fopen(filename, MODE_READ)))
+  if (!(file = openFile(filename, MODE_READ)))
     return NULL;
 
   buffer = checked_calloc(1);	/* start with valid, but empty text buffer */
 
-  while (!feof(file) && num_lines < max_lines)
+  while (!checkEndOfFile(file) && num_lines < max_lines)
   {
     char line[MAX_LINE_LEN];
 
     /* read next line of input file */
-    if (!fgets(line, MAX_LINE_LEN, file))
+    if (!getStringFromFile(file, line, MAX_LINE_LEN))
       break;
 
     buffer = checked_realloc(buffer, strlen(buffer) + strlen(line) + 1);
@@ -477,39 +349,11 @@ char *GetTextBufferFromFile(char *filename, int max_lines)
     num_lines++;
   }
 
-  fclose(file);
+  closeFile(file);
 
   return buffer;
 }
 
-void DrawTextToTextArea_OLD(int x, int y, char *text, int font_nr, int line_length,
-			    int area_xsize, int area_ysize, int mask_mode)
-{
-  int area_line = 0;
-  int font_height = getFontHeight(font_nr);
-
-  if (text == NULL)
-    return;
-
-  while (*text && area_line < area_ysize)
-  {
-    char buffer[MAX_OUTPUT_LINESIZE + 1];
-    int i;
-
-    for (i = 0; i < line_length && *text; i++)
-      if ((buffer[i] = *text++) == '\n')
-	break;
-    buffer[MIN(i, area_xsize)] = '\0';
-
-    DrawTextExt(drawto, x, y + area_line * font_height, buffer, font_nr,
-		mask_mode);
-
-    area_line++;
-  }
-
-  redraw_mask |= REDRAW_FIELD;
-}
-
 static boolean RenderLineToBuffer(char **src_buffer_ptr, char *dst_buffer,
 				  int *dst_buffer_len, int line_length,
 				  boolean last_line_was_empty)
@@ -589,138 +433,6 @@ static boolean RenderLineToBuffer(char **src_buffer_ptr, char *dst_buffer,
   return buffer_filled;
 }
 
-#if 0
-void DrawTextWrapped_OLD(int x, int y, char *text, int font_nr, int line_length,
-			 int max_lines)
-{
-  char *text_ptr = text;
-  int current_line = 0;
-  int font_height = getFontHeight(font_nr);
-
-  while (*text_ptr && current_line < max_lines)
-  {
-    char buffer[line_length + 1];
-    int buffer_len = 0;
-
-    buffer[0] = '\0';
-
-    RenderLineToBuffer(&text_ptr, buffer, &buffer_len, line_length, TRUE);
-
-    DrawText(x, y + current_line * font_height, buffer, font_nr);
-    current_line++;
-  }
-}
-#endif
-
-#if 0
-int DrawTextFromFile_OLD(int x, int y, char *filename, int font_nr,
-			 int line_length, int max_lines, boolean wrap_text)
-{
-  int font_height = getFontHeight(font_nr);
-  char line[MAX_LINE_LEN];
-  char buffer[line_length + 1];
-  int buffer_len;
-  int current_line = 0;
-  FILE *file;
-
-  if (current_line >= max_lines)
-    return 0;
-
-  if (filename == NULL)
-    return 0;
-
-  if (!(file = fopen(filename, MODE_READ)))
-    return 0;
-
-  buffer[0] = '\0';
-  buffer_len = 0;
-
-  while (!feof(file) && current_line < max_lines)
-  {
-    char *line_ptr;
-    boolean last_line_was_empty = TRUE;
-
-    /* read next line of input file */
-    if (!fgets(line, MAX_LINE_LEN, file))
-      break;
-
-    /* skip comments (lines directly beginning with '#') */
-    if (line[0] == '#')
-      continue;
-
-    /* cut trailing newline from input line */
-    for (line_ptr = line; *line_ptr; line_ptr++)
-    {
-      if (*line_ptr == '\n' || *line_ptr == '\r')
-      {
-	*line_ptr = '\0';
-	break;
-      }
-    }
-
-    if (strlen(line) == 0)		/* special case: force empty line */
-      strcpy(line, "\n");
-
-    line_ptr = line;
-
-    while (*line_ptr && current_line < max_lines)
-    {
-#if 1
-      boolean buffer_filled;
-
-      if (wrap_text)
-      {
-	buffer_filled = RenderLineToBuffer(&line_ptr, buffer, &buffer_len,
-					   line_length, last_line_was_empty);
-      }
-      else
-      {
-	if (strlen(line_ptr) <= line_length)
-	{
-	  buffer_len = strlen(line_ptr);
-	  strcpy(buffer, line_ptr);
-	}
-	else
-	{
-	  buffer_len = line_length;
-	  strncpy(buffer, line_ptr, line_length);
-	}
-
-	buffer[buffer_len] = '\0';
-	line_ptr += buffer_len;
-
-	buffer_filled = TRUE;
-      }
-#else
-      boolean buffer_filled = RenderLineToBuffer(&line_ptr, buffer, &buffer_len,
-						 line_length, last_line_was_empty);
-#endif
-
-      if (buffer_filled)
-      {
-	DrawText(x, y + current_line * font_height, buffer, font_nr);
-	current_line++;
-
-	last_line_was_empty = (buffer_len == 0);
-
-	buffer[0] = '\0';
-	buffer_len = 0;
-      }
-    }
-  }
-
-  fclose(file);
-
-  if (buffer_len > 0 && current_line < max_lines)
-  {
-    DrawText(x, y + current_line * font_height, buffer, font_nr);
-    current_line++;
-  }
-
-  return current_line;
-}
-#endif
-
 static boolean getCheckedTokenValueFromString(char *string, char **token,
 					      char **value)
 {
@@ -771,10 +483,6 @@ int DrawTextBuffer(int x, int y, char *text_buffer, int font_nr,
 		   int line_spacing, int mask_mode, boolean autowrap,
 		   boolean centered, boolean parse_comments)
 {
-#if 0
-  int font_width = getFontWidth(font_nr);
-  int font_height = getFontHeight(font_nr);
-#endif
   char buffer[line_length + 1];
   int buffer_len;
   int current_line = 0;
@@ -796,11 +504,7 @@ int DrawTextBuffer(int x, int y, char *text_buffer, int font_nr,
     char line[MAX_LINE_LEN + 1];
     char *line_ptr;
     boolean last_line_was_empty = TRUE;
-#if 1
     int num_line_chars = MAX_LINE_LEN;
-#else
-    int num_line_chars = (autowrap ? MAX_LINE_LEN : line_length);
-#endif
     int i;
 
     /* copy next line from text buffer to line buffer (nearly fgets() style) */
@@ -891,25 +595,8 @@ int DrawTextBuffer(int x, int y, char *text_buffer, int font_nr,
 
       if (buffer_filled)
       {
-#if 1
 	DrawTextBuffer_Flush(x, y, buffer, font_nr, line_length, cut_length,
 			     line_spacing, mask_mode, centered, current_line);
-#else
-	int offset_chars = (centered ? (line_length - buffer_len) / 2 : 0);
-	int offset_xsize =
-	  (centered ?  font_width * (line_length - buffer_len) / 2 : 0);
-	int final_cut_length = MAX(0, cut_length - offset_chars);
-	int xx = x + offset_xsize;
-
-	buffer[final_cut_length] = '\0';
-
-	if (mask_mode != -1)
-	  DrawTextExt(drawto, xx, y + current_line * font_height, buffer,
-		      font_nr, mask_mode);
-	else
-	  DrawText(xx, y + current_line * font_height, buffer, font_nr);
-#endif
-
 	current_line++;
 
 	last_line_was_empty = (buffer_len == 0);
@@ -922,31 +609,32 @@ int DrawTextBuffer(int x, int y, char *text_buffer, int font_nr,
 
   if (buffer_len > 0 && current_line < max_lines)
   {
-#if 1
     DrawTextBuffer_Flush(x, y, buffer, font_nr, line_length, cut_length,
 			 line_spacing, mask_mode, centered, current_line);
-#else
-    int offset_chars = (centered ? (line_length - buffer_len) / 2 : 0);
-	int offset_xsize =
-	  (centered ?  font_width * (line_length - buffer_len) / 2 : 0);
-    int final_cut_length = MAX(0, cut_length - offset_chars);
-    int xx = x + offset_xsize;
-
-    buffer[final_cut_length] = '\0';
-
-    if (mask_mode != -1)
-      DrawTextExt(drawto, xx, y + current_line * font_height, buffer,
-		  font_nr, mask_mode);
-    else
-      DrawText(xx, y + current_line * font_height, buffer, font_nr);
-#endif
-
     current_line++;
   }
 
   return current_line;
 }
 
+int DrawTextBufferVA(int x, int y, char *format, va_list ap, int font_nr,
+		     int line_length, int cut_length, int max_lines,
+		     int line_spacing, int mask_mode, boolean autowrap,
+		     boolean centered, boolean parse_comments)
+{
+  char text_buffer[MAX_OUTPUT_LINESIZE];
+  int text_length = vsnprintf(text_buffer, MAX_OUTPUT_LINESIZE, format, ap);
+
+  if (text_length >= MAX_OUTPUT_LINESIZE)
+    Error(ERR_WARN, "string too long in DrawTextBufferVA() -- truncated");
+
+  int num_lines_printed = DrawTextBuffer(x, y, text_buffer, font_nr,
+					 line_length, cut_length, max_lines,
+					 line_spacing, mask_mode, autowrap,
+					 centered, parse_comments);
+  return num_lines_printed;
+}
+
 int DrawTextFile(int x, int y, char *filename, int font_nr,
 		 int line_length, int cut_length, int max_lines,
 		 int line_spacing, int mask_mode, boolean autowrap,
@@ -961,19 +649,3 @@ int DrawTextFile(int x, int y, char *filename, int font_nr,
 
   return num_lines_printed;
 }
-
-#if 0
-void DrawTextWrapped(int x, int y, char *text, int font_nr, int line_length,
-		     int max_lines)
-{
-  DrawTextBuffer(x, y, text, font_nr, line_length, -1, max_lines, -1, TRUE,
-		 FALSE, FALSE);
-}
-
-void DrawTextToTextArea(int x, int y, char *text, int font_nr, int line_length,
-			int cut_length, int max_lines, int mask_mode)
-{
-  DrawTextBuffer(x, y, text, font_nr, line_length, cut_length, max_lines,
-		 mask_mode, FALSE, FALSE, FALSE);
-}
-#endif
