@@ -1,15 +1,13 @@
-/***********************************************************
-* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
-*----------------------------------------------------------*
-* (c) 1995-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* main.c                                                   *
-***********************************************************/
+// ============================================================================
+// Rocks'n'Diamonds - McDuffin Strikes Back!
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// main.c
+// ============================================================================
 
 #include "libgame/libgame.h"
 
@@ -17,19 +15,22 @@
 #include "init.h"
 #include "game.h"
 #include "tape.h"
+#include "tools.h"
+#include "files.h"
 #include "events.h"
 #include "config.h"
 
-Bitmap		       *bitmap_db_store;
-Bitmap		       *bitmap_db_cross;
 Bitmap		       *bitmap_db_field;
 Bitmap		       *bitmap_db_panel;
-Bitmap		       *bitmap_db_door;
-Bitmap		       *bitmap_db_toons;
+Bitmap		       *bitmap_db_door_1;
+Bitmap		       *bitmap_db_door_2;
+Bitmap		       *bitmap_db_store_1;
+Bitmap		       *bitmap_db_store_2;
 DrawBuffer	       *fieldbuffer;
 DrawBuffer	       *drawto_field;
 
 int			game_status = -1;
+boolean			game_status_last_screen = -1;
 boolean			level_editor_test_game = FALSE;
 boolean			network_playing = FALSE;
 
@@ -40,17 +41,6 @@ SDL_Thread	       *server_thread;
 
 int			key_joystick_mapping = 0;
 
-#if 1
-#if NEW_SCROLL
-boolean			redraw[2 + MAX_LEV_FIELDX + 2][2 + MAX_LEV_FIELDY + 2];
-#else
-boolean			redraw[MAX_LEV_FIELDX + 2][MAX_LEV_FIELDY + 2];
-#endif
-#else
-boolean			redraw[MAX_BUF_XSIZE][MAX_BUF_YSIZE];
-#endif
-int			redraw_x1 = 0, redraw_y1 = 0;
-
 short			Feld[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
 short			MovPos[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
 short			MovDir[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
@@ -93,38 +83,46 @@ int			ActiveFont[NUM_FONTS];
 int			lev_fieldx, lev_fieldy;
 int			scroll_x, scroll_y;
 
-int			WIN_XSIZE = 672, WIN_YSIZE = 560;
-int			SCR_FIELDX = 17, SCR_FIELDY = 17;
-int			SX = 8, SY = 8;
+int			WIN_XSIZE = WIN_XSIZE_DEFAULT;
+int			WIN_YSIZE = WIN_YSIZE_DEFAULT;
+
+int			SCR_FIELDX = SCR_FIELDX_DEFAULT;
+int			SCR_FIELDY = SCR_FIELDY_DEFAULT;
+
 int			REAL_SX = 6, REAL_SY = 6;
+int			SX = 8, SY = 8;
 int			DX = 566, DY = 60;
 int			VX = 566, VY = 400;
 int			EX = 566, EY = 356;
 int			dDX, dDY;
-int			SXSIZE = 17 * TILEX;	/* SCR_FIELDX * TILEX */
-int			SYSIZE = 17 * TILEY;	/* SCR_FIELDY * TILEY */
-int			FULL_SXSIZE = 2 + 17 * TILEX + 2; /* 2 + SXSIZE + 2 */
-int			FULL_SYSIZE = 2 + 17 * TILEY + 2; /* 2 + SYSIZE + 2 */
+
+int			FULL_SXSIZE = 2 + SXSIZE_DEFAULT + 2;
+int			FULL_SYSIZE = 2 + SYSIZE_DEFAULT + 2;
+int			SXSIZE = SXSIZE_DEFAULT;
+int			SYSIZE = SYSIZE_DEFAULT;
+
+int			FADE_SX = 6, FADE_SY = 6;
+int			FADE_SXSIZE = 2 + SXSIZE_DEFAULT + 2;
+int			FADE_SYSIZE = 2 + SXSIZE_DEFAULT + 2;
+
+int			DXSIZE = 100;
+int			DYSIZE = 280;
+int			VXSIZE = 100;
+int			VYSIZE = 100;
+int			EXSIZE = 100;
+int			EYSIZE = 144;
 int			TILESIZE_VAR = TILESIZE;
 
-#if 1
 int			FX, FY;
-#else
-int			FX = SX, FY = SY;
-#endif
 int			ScrollStepSize;
 int			ScreenMovDir = MV_NONE, ScreenMovPos = 0;
 int			ScreenGfxPos = 0;
 int			BorderElement = EL_STEELWALL;
+int			MenuFrameDelay = MENU_FRAME_DELAY;
 int			GameFrameDelay = GAME_FRAME_DELAY;
 int			FfwdFrameDelay = FFWD_FRAME_DELAY;
-#if 1
 int			BX1, BY1;
 int			BX2, BY2;
-#else
-int			BX1 = 0, BY1 = 0;
-int			BX2 = SCR_FIELDX - 1, BY2 = SCR_FIELDY - 1;
-#endif
 int			SBX_Left, SBX_Right;
 int			SBY_Upper, SBY_Lower;
 int			ZX, ZY;
@@ -146,22 +144,40 @@ struct GlobalInfo	global;
 struct BorderInfo	border;
 struct ViewportInfo	viewport;
 struct TitleFadingInfo	fading;
+struct TitleFadingInfo	title_initial_first_default;
 struct TitleFadingInfo	title_initial_default;
+struct TitleFadingInfo	title_first_default;
 struct TitleFadingInfo	title_default;
+struct TitleMessageInfo	titlescreen_initial_first_default;
+struct TitleMessageInfo	titlescreen_initial_first[MAX_NUM_TITLE_IMAGES];
+struct TitleMessageInfo	titlescreen_initial_default;
+struct TitleMessageInfo	titlescreen_initial[MAX_NUM_TITLE_IMAGES];
+struct TitleMessageInfo	titlescreen_first_default;
+struct TitleMessageInfo	titlescreen_first[MAX_NUM_TITLE_IMAGES];
+struct TitleMessageInfo	titlescreen_default;
+struct TitleMessageInfo	titlescreen[MAX_NUM_TITLE_IMAGES];
+struct TitleMessageInfo	titlemessage_initial_first_default;
+struct TitleMessageInfo	titlemessage_initial_first[MAX_NUM_TITLE_MESSAGES];
 struct TitleMessageInfo	titlemessage_initial_default;
 struct TitleMessageInfo	titlemessage_initial[MAX_NUM_TITLE_MESSAGES];
+struct TitleMessageInfo	titlemessage_first_default;
+struct TitleMessageInfo	titlemessage_first[MAX_NUM_TITLE_MESSAGES];
 struct TitleMessageInfo	titlemessage_default;
 struct TitleMessageInfo	titlemessage[MAX_NUM_TITLE_MESSAGES];
 struct TitleMessageInfo	readme;
 struct InitInfo		init, init_last;
 struct MenuInfo		menu;
 struct DoorInfo		door_1, door_2;
+struct RequestInfo	request;
 struct PreviewInfo	preview;
+struct EditorInfo	editor;
+
 struct GraphicInfo     *graphic_info = NULL;
 struct SoundInfo       *sound_info = NULL;
 struct MusicInfo       *music_info = NULL;
 struct MusicFileInfo   *music_file_info = NULL;
 struct HelpAnimInfo    *helpanim_info = NULL;
+
 SetupFileHash          *helptext_info = NULL;
 SetupFileHash	       *image_config_hash = NULL;
 SetupFileHash	       *element_token_hash = NULL;
@@ -1101,37 +1117,37 @@ struct ElementNameInfo element_name_info[MAX_NUM_ELEMENTS + 1] =
   {
     "char_copyright",
     "char",
-    "letter ''"
+    "letter '\xa9'"
   },
   {
     "char_aumlaut",
     "char",
-    "letter ''"
+    "letter '\xc4'"
   },
   {
     "char_oumlaut",
     "char",
-    "letter ''"
+    "letter '\xd6'"
   },
   {
     "char_uumlaut",
     "char",
-    "letter ''"
+    "letter '\xdc'"
   },
   {
     "char_degree",
     "char",
-    "letter ''"
+    "letter '\xb0'"
   },
   {
     "char_trademark",
     "char",
-    "letter ''"
+    "letter '\xae'"
   },
   {
     "char_cursor",
     "char",
-    "letter ''"
+    "letter '\xa0'"
   },
   {
     "char_unused",
@@ -4161,37 +4177,37 @@ struct ElementNameInfo element_name_info[MAX_NUM_ELEMENTS + 1] =
   {
     "steel_char_copyright",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xa9'"
   },
   {
     "steel_char_aumlaut",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xc4'"
   },
   {
     "steel_char_oumlaut",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xd6'"
   },
   {
     "steel_char_uumlaut",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xdc'"
   },
   {
     "steel_char_degree",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xb0'"
   },
   {
     "steel_char_trademark",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xae'"
   },
   {
     "steel_char_cursor",
     "steel_char",
-    "steel letter ''"
+    "steel letter '\xa0'"
   },
   {
     "steel_char_unused",
@@ -5430,6 +5446,38 @@ struct ElementActionInfo element_action_info[NUM_ACTIONS + 1 + 1] =
   { ".page[30]",		ACTION_PAGE_30,			FALSE	},
   { ".page[31]",		ACTION_PAGE_31,			FALSE	},
   { ".page[32]",		ACTION_PAGE_32,			FALSE	},
+  { ".part_1",			ACTION_PART_1,			FALSE	},
+  { ".part_2",			ACTION_PART_2,			FALSE	},
+  { ".part_3",			ACTION_PART_3,			FALSE	},
+  { ".part_4",			ACTION_PART_4,			FALSE	},
+  { ".part_5",			ACTION_PART_5,			FALSE	},
+  { ".part_6",			ACTION_PART_6,			FALSE	},
+  { ".part_7",			ACTION_PART_7,			FALSE	},
+  { ".part_8",			ACTION_PART_8,			FALSE	},
+  { ".part_9",			ACTION_PART_9,			FALSE	},
+  { ".part_10",			ACTION_PART_10,			FALSE	},
+  { ".part_11",			ACTION_PART_11,			FALSE	},
+  { ".part_12",			ACTION_PART_12,			FALSE	},
+  { ".part_13",			ACTION_PART_13,			FALSE	},
+  { ".part_14",			ACTION_PART_14,			FALSE	},
+  { ".part_15",			ACTION_PART_15,			FALSE	},
+  { ".part_16",			ACTION_PART_16,			FALSE	},
+  { ".part_17",			ACTION_PART_17,			FALSE	},
+  { ".part_18",			ACTION_PART_18,			FALSE	},
+  { ".part_19",			ACTION_PART_19,			FALSE	},
+  { ".part_20",			ACTION_PART_20,			FALSE	},
+  { ".part_21",			ACTION_PART_21,			FALSE	},
+  { ".part_22",			ACTION_PART_22,			FALSE	},
+  { ".part_23",			ACTION_PART_23,			FALSE	},
+  { ".part_24",			ACTION_PART_24,			FALSE	},
+  { ".part_25",			ACTION_PART_25,			FALSE	},
+  { ".part_26",			ACTION_PART_26,			FALSE	},
+  { ".part_27",			ACTION_PART_27,			FALSE	},
+  { ".part_28",			ACTION_PART_28,			FALSE	},
+  { ".part_29",			ACTION_PART_29,			FALSE	},
+  { ".part_30",			ACTION_PART_30,			FALSE	},
+  { ".part_31",			ACTION_PART_31,			FALSE	},
+  { ".part_32",			ACTION_PART_32,			FALSE	},
   { ".other",			ACTION_OTHER,			FALSE	},
 
   /* empty suffix always matches -- check as last entry in InitSoundInfo() */
@@ -5457,7 +5505,17 @@ struct SpecialSuffixInfo special_suffix_info[NUM_SPECIAL_GFX_ARGS + 1 + 1] =
   { ".[DEFAULT]",		GFX_SPECIAL_ARG_DEFAULT,		},
   { ".LOADING",			GFX_SPECIAL_ARG_LOADING,		},
   { ".TITLE_INITIAL",		GFX_SPECIAL_ARG_TITLE_INITIAL,		},
+  { ".TITLE_INITIAL_1",		GFX_SPECIAL_ARG_TITLE_INITIAL_1,	},
+  { ".TITLE_INITIAL_2",		GFX_SPECIAL_ARG_TITLE_INITIAL_2,	},
+  { ".TITLE_INITIAL_3",		GFX_SPECIAL_ARG_TITLE_INITIAL_3,	},
+  { ".TITLE_INITIAL_4",		GFX_SPECIAL_ARG_TITLE_INITIAL_4,	},
+  { ".TITLE_INITIAL_5",		GFX_SPECIAL_ARG_TITLE_INITIAL_5,	},
   { ".TITLE",			GFX_SPECIAL_ARG_TITLE,			},
+  { ".TITLE_1",			GFX_SPECIAL_ARG_TITLE_1,		},
+  { ".TITLE_2",			GFX_SPECIAL_ARG_TITLE_2,		},
+  { ".TITLE_3",			GFX_SPECIAL_ARG_TITLE_3,		},
+  { ".TITLE_4",			GFX_SPECIAL_ARG_TITLE_4,		},
+  { ".TITLE_5",			GFX_SPECIAL_ARG_TITLE_5,		},
   { ".MAIN",			GFX_SPECIAL_ARG_MAIN,			},
   { ".LEVELS",			GFX_SPECIAL_ARG_LEVELS			},
   { ".LEVELNR",			GFX_SPECIAL_ARG_LEVELNR			},
@@ -5471,6 +5529,15 @@ struct SpecialSuffixInfo special_suffix_info[NUM_SPECIAL_GFX_ARGS + 1 + 1] =
   { ".PANEL",			GFX_SPECIAL_ARG_PANEL,			},
   { ".PREVIEW",			GFX_SPECIAL_ARG_PREVIEW,		},
   { ".CRUMBLED",		GFX_SPECIAL_ARG_CRUMBLED,		},
+  { ".MAINONLY",		GFX_SPECIAL_ARG_MAINONLY,		},
+  { ".TYPENAME",		GFX_SPECIAL_ARG_TYPENAME,		},
+  { ".SUBMENU",			GFX_SPECIAL_ARG_SUBMENU,		},
+  { ".MENU",			GFX_SPECIAL_ARG_MENU,			},
+  { ".TOONS",			GFX_SPECIAL_ARG_TOONS,			},
+  { ".SCORESOLD",		GFX_SPECIAL_ARG_SCORESOLD,		},
+  { ".SCORESNEW",		GFX_SPECIAL_ARG_SCORESNEW,		},
+  { ".FADING",			GFX_SPECIAL_ARG_FADING,			},
+  { ".QUIT",			GFX_SPECIAL_ARG_QUIT,			},
 
   /* empty suffix always matches -- check as last entry in InitMusicInfo() */
   { "",				GFX_SPECIAL_ARG_DEFAULT,		},
@@ -5513,15 +5580,19 @@ struct FontInfo font_info[NUM_FONTS + 1] =
   { "font.envelope_2"		},
   { "font.envelope_3"		},
   { "font.envelope_4"		},
+  { "font.request"		},
   { "font.input_1.active"	},
   { "font.input_2.active"	},
   { "font.input_1"		},
   { "font.input_2"		},
+  { "font.option_off_narrow"	},
   { "font.option_off"		},
+  { "font.option_on_narrow"	},
   { "font.option_on"		},
   { "font.value_1"		},
   { "font.value_2"		},
   { "font.value_old"		},
+  { "font.value_narrow"		},
   { "font.level_number.active"	},
   { "font.level_number"		},
   { "font.tape_recorder"	},
@@ -5532,6 +5603,82 @@ struct FontInfo font_info[NUM_FONTS + 1] =
   { NULL			}
 };
 
+struct GlobalAnimInfo global_anim_info[NUM_GLOBAL_ANIM_TOKENS + 1];
+
+/* this contains predefined structure elements to init "global_anim_info" */
+struct GlobalAnimNameInfo global_anim_name_info[NUM_GLOBAL_ANIM_TOKENS + 1] =
+{
+  /* (real) graphic definitions used to define animation graphics */
+  { "gfx.global.anim_1",	},
+  { "gfx.global.anim_2",	},
+  { "gfx.global.anim_3",	},
+  { "gfx.global.anim_4",	},
+  { "gfx.global.anim_5",	},
+  { "gfx.global.anim_6",	},
+  { "gfx.global.anim_7",	},
+  { "gfx.global.anim_8",	},
+  { "gfx.global.anim_9",	},
+  { "gfx.global.anim_10",	},
+  { "gfx.global.anim_11",	},
+  { "gfx.global.anim_12",	},
+  { "gfx.global.anim_13",	},
+  { "gfx.global.anim_14",	},
+  { "gfx.global.anim_15",	},
+  { "gfx.global.anim_16",	},
+  { "gfx.global.anim_17",	},
+  { "gfx.global.anim_18",	},
+  { "gfx.global.anim_19",	},
+  { "gfx.global.anim_20",	},
+  { "gfx.global.anim_21",	},
+  { "gfx.global.anim_22",	},
+  { "gfx.global.anim_23",	},
+  { "gfx.global.anim_24",	},
+  { "gfx.global.anim_25",	},
+  { "gfx.global.anim_26",	},
+  { "gfx.global.anim_27",	},
+  { "gfx.global.anim_28",	},
+  { "gfx.global.anim_29",	},
+  { "gfx.global.anim_30",	},
+  { "gfx.global.anim_31",	},
+  { "gfx.global.anim_32",	},
+
+  /* (dummy) graphic definitions used to define animation controls */
+  { "global.anim_1",		},
+  { "global.anim_2",		},
+  { "global.anim_3",		},
+  { "global.anim_4",		},
+  { "global.anim_5",		},
+  { "global.anim_6",		},
+  { "global.anim_7",		},
+  { "global.anim_8",		},
+  { "global.anim_9",		},
+  { "global.anim_10",		},
+  { "global.anim_11",		},
+  { "global.anim_12",		},
+  { "global.anim_13",		},
+  { "global.anim_14",		},
+  { "global.anim_15",		},
+  { "global.anim_16",		},
+  { "global.anim_17",		},
+  { "global.anim_18",		},
+  { "global.anim_19",		},
+  { "global.anim_20",		},
+  { "global.anim_21",		},
+  { "global.anim_22",		},
+  { "global.anim_23",		},
+  { "global.anim_24",		},
+  { "global.anim_25",		},
+  { "global.anim_26",		},
+  { "global.anim_27",		},
+  { "global.anim_28",		},
+  { "global.anim_29",		},
+  { "global.anim_30",		},
+  { "global.anim_31",		},
+  { "global.anim_32",		},
+
+  { NULL			}
+};
+
 
 /* ------------------------------------------------------------------------- */
 /* music token prefix definitions                                            */
@@ -5551,53 +5698,153 @@ struct MusicPrefixInfo music_prefix_info[NUM_MUSIC_PREFIXES + 1] =
 
 static void print_usage()
 {
-  printf("\n"
-	 "Usage: %s [OPTION]... [HOSTNAME [PORT]]\n"
-	 "\n"
-	 "Options:\n"
-	 "  -d, --display HOSTNAME[:SCREEN]  specify X server display\n"
-	 "  -b, --basepath DIRECTORY         alternative base DIRECTORY\n"
-	 "  -l, --level DIRECTORY            alternative level DIRECTORY\n"
-	 "  -g, --graphics DIRECTORY         alternative graphics DIRECTORY\n"
-	 "  -s, --sounds DIRECTORY           alternative sounds DIRECTORY\n"
-	 "  -m, --music DIRECTORY            alternative music DIRECTORY\n"
-	 "  -n, --network                    network multiplayer game\n"
-	 "      --serveronly                 only start network server\n"
-	 "  -v, --verbose                    verbose mode\n"
-	 "      --debug                      display debugging information\n"
-	 "      --debug-x11-sync             enable X11 synchronous mode\n"
-	 "  -e, --execute COMMAND            execute batch COMMAND\n"
-	 "\n"
-	 "Valid commands for '--execute' option:\n"
-	 "  \"print graphicsinfo.conf\"        print default graphics config\n"
-	 "  \"print soundsinfo.conf\"          print default sounds config\n"
-	 "  \"print musicinfo.conf\"           print default music config\n"
-	 "  \"print editorsetup.conf\"         print default editor config\n"
-	 "  \"print helpanim.conf\"            print default helpanim config\n"
-	 "  \"print helptext.conf\"            print default helptext config\n"
-	 "  \"dump level FILE\"                dump level data from FILE\n"
-	 "  \"dump tape FILE\"                 dump tape data from FILE\n"
-	 "  \"autoplay LEVELDIR [NR ...]\"     play level tapes for LEVELDIR\n"
-	 "  \"convert LEVELDIR [NR]\"          convert levels in LEVELDIR\n"
-	 "  \"create images DIRECTORY\"        write BMP images to DIRECTORY\n"
-	 "\n",
-	 program.command_basename);
+  Print("\n"
+	"Usage: %s [OPTION]... [HOSTNAME [PORT]]\n"
+	"\n"
+	"Options:\n"
+	"  -b, --basepath DIRECTORY         alternative base DIRECTORY\n"
+	"  -l, --level DIRECTORY            alternative level DIRECTORY\n"
+	"  -g, --graphics DIRECTORY         alternative graphics DIRECTORY\n"
+	"  -s, --sounds DIRECTORY           alternative sounds DIRECTORY\n"
+	"  -m, --music DIRECTORY            alternative music DIRECTORY\n"
+	"      --mytapes                    use private tapes for tape tests\n"
+	"  -n, --network                    network multiplayer game\n"
+	"      --serveronly                 only start network server\n"
+	"  -v, --verbose                    verbose mode\n"
+	"  -V, --version                    show program version\n"
+	"      --debug                      display debugging information\n"
+	"  -e, --execute COMMAND            execute batch COMMAND\n"
+	"\n"
+	"Valid commands for '--execute' option:\n"
+	"  \"print graphicsinfo.conf\"        print default graphics config\n"
+	"  \"print soundsinfo.conf\"          print default sounds config\n"
+	"  \"print musicinfo.conf\"           print default music config\n"
+	"  \"print editorsetup.conf\"         print default editor config\n"
+	"  \"print helpanim.conf\"            print default helpanim config\n"
+	"  \"print helptext.conf\"            print default helptext config\n"
+	"  \"dump level FILE\"                dump level data from FILE\n"
+	"  \"dump tape FILE\"                 dump tape data from FILE\n"
+	"  \"autotest LEVELDIR [NR ...]\"     test level tapes for LEVELDIR\n"
+	"  \"autoplay LEVELDIR [NR ...]\"     play level tapes for LEVELDIR\n"
+	"  \"autoffwd LEVELDIR [NR ...]\"     ffwd level tapes for LEVELDIR\n"
+	"  \"autowarp LEVELDIR [NR ...]\"     warp level tapes for LEVELDIR\n"
+	"  \"convert LEVELDIR [NR]\"          convert levels in LEVELDIR\n"
+	"  \"create images DIRECTORY\"        write BMP images to DIRECTORY\n"
+	"  \"create CE image DIRECTORY\"      write BMP image to DIRECTORY\n"
+	"\n",
+	program.command_basename);
+}
+
+static void print_version()
+{
+  Print("%s %d.%d.%d.%d%s\n",
+	PROGRAM_TITLE_STRING,
+	PROGRAM_VERSION_MAJOR,
+	PROGRAM_VERSION_MINOR,
+	PROGRAM_VERSION_PATCH,
+	PROGRAM_VERSION_BUILD,
+	PROGRAM_VERSION_EXTRA);
+
+  if (options.debug)
+  {
+    SDL_version sdl_version;
+
+    SDL_VERSION(&sdl_version);
+    Print("- SDL %d.%d.%d\n",
+	  sdl_version.major,
+	  sdl_version.minor,
+	  sdl_version.patch);
+
+    SDL_IMAGE_VERSION(&sdl_version);
+    Print("- SDL_image %d.%d.%d\n",
+	  sdl_version.major,
+	  sdl_version.minor,
+	  sdl_version.patch);
+
+    SDL_MIXER_VERSION(&sdl_version);
+    Print("- SDL_mixer %d.%d.%d\n",
+	  sdl_version.major,
+	  sdl_version.minor,
+	  sdl_version.patch);
+
+    SDL_NET_VERSION(&sdl_version);
+    Print("- SDL_net %d.%d.%d\n",
+	  sdl_version.major,
+	  sdl_version.minor,
+	  sdl_version.patch);
+  }
+}
+
+static void InitProgramConfig(char *command_filename)
+{
+  char *program_title = PROGRAM_TITLE_STRING;
+  char *program_icon_file = PROGRAM_ICON_FILENAME;
+  char *config_filename = getProgramConfigFilename(command_filename);
+  char *userdata_basename = getBaseNameNoSuffix(command_filename);
+  char *userdata_subdir;
+  char *userdata_subdir_unix;
+
+  // read default program config, if existing
+  if (fileExists(config_filename))
+  {
+    // if program config file exists, derive Unix user data directory from it
+    userdata_basename = getBaseName(config_filename);
+
+    if (strSuffix(userdata_basename, ".conf"))
+      userdata_basename[strlen(userdata_basename) - 5] = '\0';
+
+    LoadSetupFromFilename(config_filename);
+  }
+
+  // set user data directory for Linux/Unix (but not Mac OS X)
+  userdata_subdir_unix = getStringCat2(".", userdata_basename);
+
+  // set program title from potentially redefined program title
+  if (setup.internal.program_title != NULL &&
+      strlen(setup.internal.program_title) > 0)
+    program_title = getStringCopy(setup.internal.program_title);
+
+  // set program icon file from potentially redefined program icon file
+  if (setup.internal.program_icon_file != NULL &&
+      strlen(setup.internal.program_icon_file) > 0)
+    program_icon_file = getStringCopy(setup.internal.program_icon_file);
+
+#if defined(PLATFORM_WIN32) || defined(PLATFORM_MACOSX)
+  userdata_subdir = program_title;
+#elif defined(PLATFORM_UNIX)
+  userdata_subdir = userdata_subdir_unix;
+#else
+  userdata_subdir = USERDATA_DIRECTORY_OTHER;
+#endif
+
+  // set default window size (only relevant on program startup)
+  if (setup.internal.default_window_width  != 0 &&
+      setup.internal.default_window_height != 0)
+  {
+    WIN_XSIZE = setup.internal.default_window_width;
+    WIN_YSIZE = setup.internal.default_window_height;
+  }
+
+  InitProgramInfo(command_filename,
+		  config_filename,
+		  userdata_subdir,
+		  program_title,
+		  program_title,
+		  program_icon_file,
+		  COOKIE_PREFIX,
+		  GAME_VERSION_ACTUAL);
 }
 
 int main(int argc, char *argv[])
 {
-  char * window_title_string = getWindowTitleString();
-
-  InitProgramInfo(argv[0], USERDATA_DIRECTORY, USERDATA_DIRECTORY_UNIX,
-		  PROGRAM_TITLE_STRING, window_title_string, ICON_TITLE_STRING,
-		  X11_ICON_FILENAME, X11_ICONMASK_FILENAME, SDL_ICON_FILENAME,
-		  MSDOS_POINTER_FILENAME,
-		  COOKIE_PREFIX, FILENAME_PREFIX, GAME_VERSION_ACTUAL);
+  InitProgramConfig(argv[0]);
 
+  InitWindowTitleFunction(getWindowTitleString);
+  InitExitMessageFunction(DisplayExitMessage);
   InitExitFunction(CloseAllAndExit);
   InitPlatformDependentStuff();
 
-  GetOptions(argv, print_usage);
+  GetOptions(argc, argv, print_usage, print_version);
   OpenAll();
 
   EventLoop();
